@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 
 ## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
@@ -28,12 +28,7 @@ import Queue
 import authconn
 from debug import *
 import debug
-
-_ = lambda x: x
-N_ = lambda x: x
-def set_gettext_function (fn):
-    global _
-    _ = fn
+from gettext import gettext as _
 
 ######
 ###### An asynchronous libcups API using IPP with a separate worker
@@ -45,20 +40,21 @@ def set_gettext_function (fn):
 ###
 class _IPPConnectionThread(threading.Thread):
     def __init__ (self, queue, conn, reply_handler=None, error_handler=None,
-                  auth_handler=None, host=None, port=None, encryption=None):
+                  auth_handler=None, user=None, host=None, port=None,
+                  encryption=None):
                   
         threading.Thread.__init__ (self)
         self.setDaemon (True)
         self._queue = queue
         self._conn = conn
         self.host = host
-        self._port = port
+        self.port = port
         self._encryption = encryption
         self._reply_handler = reply_handler
         self._error_handler = error_handler
         self._auth_handler = auth_handler
         self._auth_queue = Queue.Queue (1)
-        self.user = None
+        self.user = user
         self._destroyed = False
         debugprint ("+%s" % self)
 
@@ -71,12 +67,15 @@ class _IPPConnectionThread(threading.Thread):
     def run (self):
         if self.host == None:
             self.host = cups.getServer ()
-        if self._port == None:
-            self._port = cups.getPort ()
+        if self.port == None:
+            self.port = cups.getPort ()
         if self._encryption == None:
             self._encryption = cups.getEncryption ()
 
-        self.user = cups.getUser ()
+        if self.user:
+            cups.setUser (self.user)
+        else:
+            self.user = cups.getUser ()
 
         try:
             cups.setPasswordCB2 (self._auth)
@@ -86,7 +85,7 @@ class _IPPConnectionThread(threading.Thread):
 
         try:
             conn = cups.Connection (host=self.host,
-                                    port=self._port,
+                                    port=self.port,
                                     encryption=self._encryption)
             self._reply (None)
         except RuntimeError, e:
@@ -96,6 +95,7 @@ class _IPPConnectionThread(threading.Thread):
         while True:
             # Wait to find out what operation to try.
             debugprint ("Awaiting further instructions")
+            self.idle = self._queue.empty ()
             item = self._queue.get ()
             debugprint ("Next task: %s" % repr (item))
             if item == None:
@@ -103,6 +103,7 @@ class _IPPConnectionThread(threading.Thread):
                 self._queue.task_done ()
                 break
 
+            self.idle = False
             (fn, args, kwds, rh, eh, ah) = item
             if rh != False:
                 self._reply_handler = rh
@@ -124,7 +125,7 @@ class _IPPConnectionThread(threading.Thread):
 
                 try:
                     conn = cups.Connection (host=self.host,
-                                            port=self._port,
+                                            port=self.port,
                                             encryption=self._encryption)
                     debugprint ("...reconnected")
 
@@ -232,8 +233,8 @@ class IPPConnection:
     """
 
     def __init__ (self, reply_handler=None, error_handler=None,
-                  auth_handler=None, host=None, port=None, encryption=None,
-                  parent=None):
+                  auth_handler=None, user=None, host=None, port=None,
+                  encryption=None, parent=None):
         debugprint ("New IPPConnection")
         self._parent = parent
         self.queue = Queue.Queue ()
@@ -241,7 +242,7 @@ class IPPConnection:
                                             reply_handler=reply_handler,
                                             error_handler=error_handler,
                                             auth_handler=auth_handler,
-                                            host=host, port=port,
+                                            user=user, host=host, port=port,
                                             encryption=encryption)
         self.thread.start ()
 
@@ -264,13 +265,21 @@ class IPPConnection:
 
     def destroy (self):
         debugprint ("DESTROY: %s" % self)
+        for binding in self.bindings:
+            delattr (self, binding)
+
         if self.thread.isAlive ():
+            gobject.timeout_add_seconds (1, self._reap_thread)
+
+    def _reap_thread (self):
+        if self.thread.idle:
             debugprint ("Putting None on the task queue")
             self.queue.put (None)
             self.queue.join ()
+            return False
 
-        for binding in self.bindings:
-            delattr (self, binding)
+        debugprint ("Thread %s still processing tasks" % self.thread)
+        return True
 
     def set_auth_info (self, password):
         """Call this from your auth_handler function."""
@@ -324,6 +333,18 @@ class _IPPAuthOperation:
         self._client_kwds = kwds
         self._client_reply_handler = reply_handler
         self._client_error_handler = error_handler
+
+        if user:
+            host = conn.thread.host
+            port = conn.thread.port
+            creds = authconn.global_authinfocache.lookup_auth_info (host=host,
+                                                                    port=port)
+            if creds:
+                if creds[0] == user:
+                    self._use_password = creds[1]
+                    self._reconnected = True
+                del creds
+
         debugprint ("+%s" % self)
 
     def __del__ (self):
@@ -559,12 +580,20 @@ class IPPAuthConnection(IPPConnection):
         self.try_as_root = try_as_root
         self.semantic = semantic
 
+        user = None
+        creds = authconn.global_authinfocache.lookup_auth_info (host=host,
+                                                                port=port)
+        if creds:
+            if creds[0] != 'root' or try_as_root:
+                user = creds[0]
+            del creds
+
         # The "connect" operation.
         op = _IPPAuthOperation (reply_handler, error_handler, self)
         IPPConnection.__init__ (self, reply_handler=reply_handler,
                                 error_handler=op.error_handler,
-                                auth_handler=op.auth_handler, host=host,
-                                port=port, encryption=encryption)
+                                auth_handler=op.auth_handler, user=user,
+                                host=host, port=port, encryption=encryption)
 
     def destroy (self):
         del self.semantic
