@@ -54,11 +54,7 @@ except ImportError:
     USE_KEYRING=False
 
 from gettext import gettext as _
-DOMAIN="system-config-printer"
-gettext.textdomain (DOMAIN)
 from statereason import StateReason
-statereason.set_gettext_function (_)
-errordialogs.set_gettext_function (_)
 
 pkgdata = config.pkgdatadir
 ICON="printer"
@@ -163,11 +159,23 @@ class PrinterURIIndex:
         return name
 
 
-class CancelJobsOperation:
-    def __init__ (self, jobviewer, jobids, purge_job):
-        self.jobviewer = jobviewer
+class CancelJobsOperation(gobject.GObject):
+    __gsignals__ = {
+        'destroy':     (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
+        'job-deleted': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                        [gobject.TYPE_INT]),
+        'ipp-error':   (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                        [gobject.TYPE_INT, gobject.TYPE_PYOBJECT]),
+        'finished':    (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
+        }
+
+    def __init__ (self, parent, host, port, encryption, jobids, purge_job):
+        gobject.GObject.__init__ (self)
         self.jobids = list (jobids)
         self.purge_job = purge_job
+        self.host = host
+        self.port = port
+        self.encryption = encryption
         if purge_job:
             if len(self.jobids) > 1:
                 dialog_title = _("Delete Jobs")
@@ -183,7 +191,7 @@ class CancelJobsOperation:
                 dialog_title = _("Cancel Job")
                 dialog_label = _("Do you really want to cancel this job?")
 
-        dialog = gtk.Dialog (dialog_title, jobviewer.JobsWindow,
+        dialog = gtk.Dialog (dialog_title, parent,
                              gtk.DIALOG_MODAL |
                              gtk.DIALOG_DESTROY_WITH_PARENT |
                              gtk.DIALOG_NO_SEPARATOR,
@@ -202,7 +210,6 @@ class CancelJobsOperation:
         label.set_alignment (0.0, 0.0)
         hbox.pack_start (label, False, False, 0)
         dialog.vbox.pack_start (hbox, False, False, 0)
-        dialog.set_data ('job-ids', self.jobids)
         dialog.connect ("response", self.on_job_cancel_prompt_response)
         dialog.connect ("delete-event", self.on_job_cancel_prompt_delete)
         dialog.show_all ()
@@ -213,15 +220,20 @@ class CancelJobsOperation:
     def __del__ (self):
         debugprint ("-%s" % self)
 
-    def destroy (self):
-        self.jobviewer = None
-        self.connection = None
+    def do_destroy (self):
+        if self.connection:
+            self.connection.destroy ()
+            self.connection = None
+
         if self.dialog:
             self.dialog.destroy ()
             self.dialog = None
 
         debugprint ("DESTROY: %s" % self)
 
+    def destroy (self):
+        self.emit ('destroy')
+
     def on_job_cancel_prompt_delete (self, dialog, event):
         self.on_job_cancel_prompt_response (dialog, gtk.RESPONSE_NO)
 
@@ -230,11 +242,16 @@ class CancelJobsOperation:
         self.dialog = None
 
         if response != gtk.RESPONSE_YES:
+            self.emit ('finished')
             return
 
-        asyncconn.Connection (host=self.jobviewer.host,
-                              port=self.jobviewer.port,
-                              encryption=self.jobviewer.encryption,
+        if len(self.jobids) == 0:
+            self.emit ('finished')
+            return
+
+        asyncconn.Connection (host=self.host,
+                              port=self.port,
+                              encryption=self.encryption,
                               reply_handler=self._connected,
                               error_handler=self._connect_failed)
 
@@ -256,39 +273,41 @@ class CancelJobsOperation:
 
     def cancelJob_error (self, connection, exc):
         debugprint ("cancelJob_error %s:%s" % (connection, repr (exc)))
-        if self.jobviewer == None:
-            return
-
         if type (exc) == cups.IPPError:
             (e, m) = exc.args
             if (e != cups.IPP_NOT_POSSIBLE and
                 e != cups.IPP_NOT_FOUND):
-                self.jobviewer.show_IPP_Error (e, m)
+                self.emit ('ipp-error', self.jobids[0], exc)
             self.cancelJob_finish(connection, None)
         else:
             self.connection._end_operation ()
             self.connection.destroy ()
-            self.jobviewer.update_monitor ()
-            raise exc
+            self.connection = None
+            self.emit ('ipp-error', self.jobids[0], exc)
+            # Give up.
+            self.emit ('finished')
+            return
 
     def cancelJob_finish (self, connection, result):
         debugprint ("cancelJob_finish %s:%s" % (connection, repr (result)))
-        if self.jobviewer == None:
-            return
-
+        self.emit ('job-deleted', self.jobids[0])
         del self.jobids[0]
         if not self.jobids:
             # Last job canceled.
             self.connection._end_operation ()
             self.connection.destroy ()
-            self.jobviewer.update_monitor ()
+            self.connection = None
+            self.emit ('finished')
+            return
         else:
             # there are other jobs to cancel/delete
             connection.cancelJob (self.jobids[0], self.purge_job,
                                   reply_handler=self.cancelJob_finish,
                                   error_handler=self.cancelJob_error)
 
-class JobViewer (GtkGUI, monitor.Watcher):
+gobject.type_register (CancelJobsOperation)
+
+class JobViewer (GtkGUI):
     required_job_attributes = set(['job-k-octets',
                                    'job-name',
                                    'job-originating-user-name',
@@ -297,16 +316,23 @@ class JobViewer (GtkGUI, monitor.Watcher):
                                    'time-at-creation',
                                    'job-preserved'])
 
+    __gsignals__ = {
+        'finished':    (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
+        }
+
     def __init__(self, bus=None, loop=None,
-                 trayicon=False, suppress_icon_hide=False,
-                 my_jobs=True, specific_dests=None, exit_handler=None,
+                 applet=False, suppress_icon_hide=False,
+                 my_jobs=True, specific_dests=None,
                  parent=None):
+        gobject.GObject.__init__ (self)
         self.loop = loop
-        self.trayicon = trayicon
+        self.applet = applet
         self.suppress_icon_hide = suppress_icon_hide
         self.my_jobs = my_jobs
         self.specific_dests = specific_dests
-        self.exit_handler = exit_handler
+        notify_caps = pynotify.get_server_caps ()
+        self.notify_has_actions = "actions" in notify_caps
+        self.notify_has_persistence = "persistence" in notify_caps
 
         self.jobs = {}
         self.jobiters = {}
@@ -320,7 +346,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.state_reason_notifications = {}
         self.auth_info_dialogs = {} # by job ID
         self.job_creation_times_timer = None
-        self.special_status_icon = False
         self.new_printer_notifications = {}
         self.completed_job_notifications = {}
         self.authenticated_jobs = set() # of job IDs
@@ -334,7 +359,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
                           "statusicon_popupmenu":
                               ["statusicon_popupmenu"]},
 
-                         domain=DOMAIN)
+                         domain=config.PACKAGE)
 
         job_action_group = gtk.ActionGroup ("JobActionGroup")
         job_action_group.add_actions ([
@@ -424,8 +449,8 @@ class JobViewer (GtkGUI, monitor.Watcher):
                  (False, True, _("Document"), self._set_job_document_text),
                  (False, True, _("Printer"), self._set_job_printer_text),
                  (False, False, _("Size"), self._set_job_size_text)]:
-            if trayicon and skip:
-                # Skip the user column for the trayicon.
+            if applet and skip:
+                # Skip the user column when running as applet.
                 continue
 
             cell = gtk.CellRendererText()
@@ -464,8 +489,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.treeview.connect ('button_release_event',
                                self.on_treeview_button_release_event)
         self.treeview.connect ('popup-menu', self.on_treeview_popup_menu)
-        self.treeview.set_has_tooltip (True)
-        self.treeview.connect ('query-tooltip', self._on_treeview_query_tooltip)
 
         self.JobsWindow.set_icon_name (ICON)
         self.JobsWindow.hide ()
@@ -488,8 +511,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
         if parent:
             self.JobsWindow.set_transient_for (parent)
 
-        self.statusbar_set = False
-
         def load_icon(theme, icon):
             try:
                 pixbuf = theme.load_icon (icon, ICON_SIZE, 0)
@@ -514,11 +535,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
                                   1.0, 1.0,
                                   gtk.gdk.INTERP_BILINEAR,
                                   127)
-        if self.trayicon:
+        if self.applet and not self.notify_has_persistence:
             self.statusicon = gtk.StatusIcon ()
-            pixbuf = load_icon (theme, ICON)
-            self.statusicon.set_from_pixbuf (pixbuf)
-            self.set_statusicon_from_pixbuf (self.icon_no_jobs)
+            self.statusicon.set_from_pixbuf (self.icon_no_jobs)
             self.statusicon.connect ('activate', self.toggle_window_display)
             self.statusicon.connect ('popup-menu', self.on_icon_popupmenu)
             self.statusicon.set_visible (False)
@@ -531,11 +550,23 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.host = cups.getServer ()
         self.port = cups.getPort ()
         self.encryption = cups.getEncryption ()
-        self.monitor = monitor.Monitor (self, bus=bus, my_jobs=my_jobs,
+        self.monitor = monitor.Monitor (bus=bus, my_jobs=my_jobs,
                                         host=self.host, port=self.port,
                                         encryption=self.encryption)
+        self.monitor.connect ('refresh', self.on_refresh)
+        self.monitor.connect ('job-added', self.job_added)
+        self.monitor.connect ('job-event', self.job_event)
+        self.monitor.connect ('job-removed', self.job_removed)
+        self.monitor.connect ('state-reason-added', self.state_reason_added)
+        self.monitor.connect ('state-reason-removed', self.state_reason_removed)
+        self.monitor.connect ('still-connecting', self.still_connecting)
+        self.monitor.connect ('now-connected', self.now_connected)
+        self.monitor.connect ('printer-added', self.printer_added)
+        self.monitor.connect ('printer-event', self.printer_event)
+        self.monitor.connect ('printer-removed', self.printer_removed)
+        self.monitor.refresh ()
 
-        if not self.trayicon:
+        if not self.applet:
             self.JobsWindow.show ()
 
         self.JobsAttributesWindow = gtk.Window()
@@ -576,6 +607,8 @@ class JobViewer (GtkGUI, monitor.Watcher):
     def cleanup (self):
         self.monitor.cleanup ()
 
+        self.JobsWindow.hide ()
+
         # Close any open notifications.
         for l in [self.new_printer_notifications.values (),
                   self.state_reason_notifications.values ()]:
@@ -596,51 +629,19 @@ class JobViewer (GtkGUI, monitor.Watcher):
         for op in self.ops:
             op.destroy ()
 
-        if self.exit_handler:
-            self.exit_handler (self)
+        if self.applet and not self.notify_has_persistence:
+            self.statusicon.set_visible (False)
+
+        self.emit ('finished')
 
     def set_process_pending (self, whether):
         self.process_pending_events = whether
 
-    # Handle "special" status icon
-    def set_special_statusicon (self, iconname, tooltip=None):
-        self.special_status_icon = True
-        self.statusicon.set_from_icon_name (iconname)
-        self.set_statusicon_visibility ()
-        if tooltip != None:
-            self.set_statusicon_tooltip (tooltip=tooltip)
-
-    def unset_special_statusicon (self):
-        self.special_status_icon = False
-        self.statusicon.set_from_pixbuf (self.saved_statusicon_pixbuf)
-        self.set_statusicon_visibility ()
-        self.set_statusicon_tooltip ()
-
-    def notify_new_printer (self, printer, notification):
-        self.new_printer_notifications[printer] = notification
-        notification.set_data ('printer-name', printer)
-        notification.connect ('closed', self.on_new_printer_notification_closed)
-        self.set_statusicon_visibility ()
-        try:
-            notification.show ()
-        except gobject.GError:
-            nonfatalException ()
-
-    def on_new_printer_notification_closed (self, notification, reason=None):
-        printer = notification.get_data ('printer-name')
-        del self.new_printer_notifications[printer]
-        self.set_statusicon_visibility ()
-
-    def set_statusicon_from_pixbuf (self, pb):
-        self.saved_statusicon_pixbuf = pb
-        if not self.special_status_icon:
-            self.statusicon.set_from_pixbuf (pb)
-
     def on_delete_event(self, *args):
-        if self.trayicon or not self.loop:
+        if self.applet or not self.loop:
             self.JobsWindow.hide ()
             self.JobsWindow.set_data ('visible', False)
-            if not self.loop:
+            if not self.applet:
                 # Being run from main app, not applet
                 self.cleanup ()
         else:
@@ -662,25 +663,31 @@ class JobViewer (GtkGUI, monitor.Watcher):
         if force_show:
             visible = False
 
-        if visible:
-            w = self.JobsWindow.window
-            aw = self.JobsAttributesWindow.window
-            (s, area, o) = self.statusicon.get_geometry ()
-            w.set_skip_taskbar_hint (True)
-            if aw != None:
-                aw.set_skip_taskbar_hint (True)
-
-            w.property_change ("_NET_WM_ICON_GEOMETRY",
-                               "CARDINAL", 32,
-                               gtk.gdk.PROP_MODE_REPLACE,
-                               list (area))
-            self.JobsWindow.iconify ()
+        if self.notify_has_persistence:
+            if visible:
+                self.JobsWindow.hide ()
+            else:
+                self.JobsWindow.show ()
         else:
-            self.JobsWindow.present ()
-            self.JobsWindow.window.set_skip_taskbar_hint (False)
-            aw = self.JobsAttributesWindow.window
-            if aw != None:
-                aw.set_skip_taskbar_hint (False)
+            if visible:
+                w = self.JobsWindow.window
+                aw = self.JobsAttributesWindow.window
+                (s, area, o) = self.statusicon.get_geometry ()
+                w.set_skip_taskbar_hint (True)
+                if aw != None:
+                    aw.set_skip_taskbar_hint (True)
+
+                w.property_change ("_NET_WM_ICON_GEOMETRY",
+                                   "CARDINAL", 32,
+                                   gtk.gdk.PROP_MODE_REPLACE,
+                                   list (area))
+                self.JobsWindow.iconify ()
+            else:
+                self.JobsWindow.present ()
+                self.JobsWindow.window.set_skip_taskbar_hint (False)
+                aw = self.JobsAttributesWindow.window
+                if aw != None:
+                    aw.set_skip_taskbar_hint (False)
 
         self.JobsWindow.set_data ('visible', not visible)
 
@@ -767,6 +774,11 @@ class JobViewer (GtkGUI, monitor.Watcher):
     def add_job (self, job, data, connection=None):
         self.update_job (job, data, connection=connection)
 
+        # There may have been an error fetching additional attributes,
+        # in which case we need to give up.
+        if not self.jobs.has_key (job):
+            return
+
         store = self.store
         iter = self.store.append (None)
         store.set_value (iter, 0, job)
@@ -877,7 +889,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.treeview.queue_draw ()
 
         # Check whether authentication is required.
-        if self.trayicon:
+        if self.applet:
             job_requires_auth = (s == cups.IPP_JOB_HELD and
                                  data.get ('job-hold-until', 'none') ==
                                  'auth-info-required')
@@ -1061,7 +1073,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         dialog.destroy ()
 
     def set_statusicon_visibility (self):
-        if not self.trayicon:
+        if not self.applet:
             return
 
         if self.suppress_icon_hide:
@@ -1081,8 +1093,10 @@ class JobViewer (GtkGUI, monitor.Watcher):
         debugprint ("num_jobs: %d" % num_jobs)
         debugprint ("num_jobs_when_hidden: %d" % self.num_jobs_when_hidden)
 
-        self.statusicon.set_visible (self.special_status_icon or
-                                     open_notifications > 0 or
+        if self.notify_has_persistence:
+            return
+
+        self.statusicon.set_visible (open_notifications > 0 or
                                      num_jobs > self.num_jobs_when_hidden)
 
         # Let the icon show/hide itself before continuing.
@@ -1221,6 +1235,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         return returncode == None
 
     def on_icon_quit_activate (self, menuitem):
+        self.cleanup ()
         if self.loop:
             self.loop.quit ()
 
@@ -1232,8 +1247,29 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
     def on_job_cancel_activate2(self, purge_job):
         if self.jobids:
-            op = CancelJobsOperation (self, self.jobids, purge_job)
+            op = CancelJobsOperation (self.JobsWindow, self.host, self.port,
+                                      self.encryption, self.jobids, purge_job)
             self.ops.append (op)
+            op.connect ('finished', self.on_canceljobs_finished)
+            op.connect ('ipp-error', self.on_canceljobs_error)
+
+    def on_canceljobs_finished (self, canceljobsoperation):
+        canceljobsoperation.destroy ()
+        i = self.ops.index (canceljobsoperation)
+        del self.ops[i]
+        self.update_monitor ()
+
+    def on_canceljobs_error (self, canceljobsoperation, jobid, exc):
+        self.update_monitor ()
+        if type (exc) == cups.IPPError:
+            (e, m) = exc.args
+            if (e != cups.IPP_NOT_POSSIBLE and
+                e != cups.IPP_NOT_FOUND):
+                self.show_IPP_Error (e, m)
+
+            return
+
+        raise exc
 
     def on_job_hold_activate(self, menuitem):
         try:
@@ -1404,14 +1440,13 @@ class JobViewer (GtkGUI, monitor.Watcher):
                                  port=self.port,
                                  encryption=self.encryption)
         except RuntimeError:
-            self.monitor.watcher.cups_connection_error (self)
             return False
 
         for jobid in self.jobids:
             if jobid not in self.jobs_attrs:
                 # add new notebook page with scrollable treeview
                 scrolledwindow = gtk.ScrolledWindow()
-                label = gtk.Label(jobid) # notebook page has label with jobid
+                label = gtk.Label(str(jobid)) # notebook page has label with jobid
                 page_index = self.notebook.append_page(scrolledwindow, label)
                 attr_treeview = gtk.TreeView()
                 scrolledwindow.add(attr_treeview)
@@ -1441,7 +1476,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
                                      port=self.port,
                                      encryption=self.encryption)
             except RuntimeError:
-                self.monitor.watcher.cups_connection_error (self)
                 return False
 
         if jobid in self.jobs_attrs:
@@ -1461,7 +1495,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
             for name, value in attrs.iteritems():
                 if name in ['job-id', 'job-printer-up-time']:
                     continue
-                attr_store.append([name, value])
+                attr_store.append([name, str(value)])
 
     def job_is_active (self, jobdata):
         state = jobdata.get ('job-state', cups.IPP_JOB_CANCELED)
@@ -1515,7 +1549,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         return pixbuf
 
     def get_icon_pixbuf (self, have_jobs=None):
-        if not self.trayicon:
+        if not self.applet:
             return
 
         if have_jobs == None:
@@ -1539,7 +1573,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         return pixbuf
 
     def set_statusicon_tooltip (self, tooltip=None):
-        if not self.trayicon:
+        if not self.applet:
             return
 
         if tooltip == None:
@@ -1585,21 +1619,31 @@ class JobViewer (GtkGUI, monitor.Watcher):
             self.worst_reason = worst_reason
             debugprint ("Worst reason: %s" % worst_reason)
 
+        self.statusbar.pop (0)
         if self.worst_reason != None:
             (title, tooltip) = self.worst_reason.get_description ()
-            if self.statusbar_set:
-                self.statusbar.pop (0)
             self.statusbar.push (0, tooltip)
-            self.statusbar_set = True
         else:
             tooltip = None
-            if self.statusbar_set:
-                self.statusbar.pop (0)
-                self.statusbar_set = False
+            status_message = ""
+            processing = 0
+            pending = 0
+            for jobid in self.active_jobs:
+                try:
+                    job_state = self.jobs[jobid]['job-state']
+                except KeyError:
+                    continue
+                if job_state == cups.IPP_JOB_PROCESSING:
+                    processing = processing + 1
+                elif job_state == cups.IPP_JOB_PENDING:
+                    pending = pending + 1
+            if ((processing > 0) or (pending > 0)):
+                status_message = _("processing / pending:   %d / %d") % (processing, pending)
+                self.statusbar.push(0, status_message)
 
-        if self.trayicon:
+        if self.applet and not self.notify_has_persistence:
             pixbuf = self.get_icon_pixbuf (have_jobs=have_jobs)
-            self.set_statusicon_from_pixbuf (pixbuf)
+            self.statusicon.set_from_pixbuf (pixbuf)
             self.set_statusicon_visibility ()
             self.set_statusicon_tooltip (tooltip=tooltip)
 
@@ -1643,7 +1687,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         notification = pynotify.Notification (title, text, 'printer')
         reason.user_notified = True
         notification.set_urgency (urgency)
-        if "actions" in pynotify.get_server_caps():
+        if self.notify_has_actions:
             notification.set_timeout (pynotify.EXPIRES_NEVER)
         notification.connect ('closed',
                               self.on_state_reason_notification_closed)
@@ -1714,43 +1758,15 @@ class JobViewer (GtkGUI, monitor.Watcher):
         del self.completed_job_notifications[jobid]
         self.set_statusicon_visibility ()
 
-    ## monitor.Watcher interface
-    def current_printers_and_jobs (self, mon, printers, jobs):
-        monitor.Watcher.current_printers_and_jobs (self, mon, printers, jobs)
-        self.set_process_pending (False)
+    ## Monitor signal handlers
+    def on_refresh (self, mon):
         self.store.clear ()
         self.jobs = {}
-        self.jobiters = {}
-        self.printer_uri_index = PrinterURIIndex (names=printers)
-        connection = None
-        for jobid, jobdata in jobs.iteritems ():
-            uri = jobdata.get ('job-printer-uri', '')
-            try:
-                printer = self.printer_uri_index.lookup (uri,
-                                                         connection=connection)
-            except KeyError:
-                printer = uri
-
-            if self.specific_dests and printer not in self.specific_dests:
-                continue
-
-            jobdata['job-printer-name'] = printer
-
-            self.add_job (jobid, jobdata, connection=connection)
-
-        self.jobs = jobs
         self.active_jobs = set()
-        for jobid, jobdata in jobs.iteritems ():
-            if (self.job_is_active (jobdata) and
-                jobdata.has_key('job-printer-name')):
-                self.active_jobs.add (jobid)
-
-        self.set_process_pending (True)
-        self.update_status ()
+        self.jobiters = {}
+        self.printer_uri_index = PrinterURIIndex ()
 
     def job_added (self, mon, jobid, eventname, event, jobdata):
-        monitor.Watcher.job_added (self, mon, jobid, eventname, event, jobdata)
-
         uri = jobdata.get ('job-printer-uri', '')
         try:
             printer = self.printer_uri_index.lookup (uri)
@@ -1767,13 +1783,17 @@ class JobViewer (GtkGUI, monitor.Watcher):
         if not self.jobiters.has_key (jobid):
             self.add_job (jobid, jobdata)
 
+        # If we failed to get required attributes for the job, bail.
+        if not self.jobiters.has_key (jobid):
+            return
+
         if self.job_is_active (jobdata):
             self.active_jobs.add (jobid)
         elif jobid in self.active_jobs:
             self.active_jobs.remove (jobid)
 
         self.update_status (have_jobs=True)
-        if self.trayicon:
+        if self.applet:
             if not self.job_is_active (jobdata):
                 return
 
@@ -1782,13 +1802,15 @@ class JobViewer (GtkGUI, monitor.Watcher):
                     self.notify_printer_state_reason_if_important (reason)
 
     def job_event (self, mon, jobid, eventname, event, jobdata):
-        monitor.Watcher.job_event (self, mon, jobid, eventname, event, jobdata)
-
         uri = jobdata.get ('job-printer-uri', '')
         try:
             printer = self.printer_uri_index.lookup (uri)
         except KeyError:
             printer = uri
+
+        if self.specific_dests and printer not in self.specific_dests:
+            return
+
         jobdata['job-printer-name'] = printer
 
         if self.job_is_active (jobdata):
@@ -1810,9 +1832,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
         jobdata = self.jobs[jobid]
 
         # If the job has finished, let the user know.
-        if self.trayicon and (eventname == 'job-completed' or
-                              (eventname == 'job-state-changed' and
-                               event['job-state'] == cups.IPP_JOB_COMPLETED)):
+        if self.applet and (eventname == 'job-completed' or
+                            (eventname == 'job-state-changed' and
+                             event['job-state'] == cups.IPP_JOB_COMPLETED)):
             reasons = event['job-state-reasons']
             if type (reasons) != list:
                 reasons = [reasons]
@@ -1827,7 +1849,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
                 self.notify_completed_job (jobid)
 
         # Look out for stopped jobs.
-        if (self.trayicon and
+        if (self.applet and
             (eventname == 'job-stopped' or
              (eventname == 'job-state-changed' and
               event['job-state'] in [cups.IPP_JOB_STOPPED,
@@ -1887,7 +1909,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
             if may_be_problem:
                 debugprint ("Problem detected")
-                self.toggle_window_display (self.statusicon, force_show=True)
+                self.toggle_window_display (None, force_show=True)
                 dialog = gtk.Dialog (_("Print Error"), self.JobsWindow, 0,
                                      (_("_Diagnose"), gtk.RESPONSE_NO,
                                         gtk.STOCK_OK, gtk.RESPONSE_OK))
@@ -1928,12 +1950,10 @@ class JobViewer (GtkGUI, monitor.Watcher):
                 dialog.show_all ()
 
     def job_removed (self, mon, jobid, eventname, event):
-        monitor.Watcher.job_removed (self, mon, jobid, eventname, event)
-
         # If the job has finished, let the user know.
-        if self.trayicon and (eventname == 'job-completed' or
-                              (eventname == 'job-state-changed' and
-                               event['job-state'] == cups.IPP_JOB_COMPLETED)):
+        if self.applet and (eventname == 'job-completed' or
+                            (eventname == 'job-state-changed' and
+                             event['job-state'] == cups.IPP_JOB_COMPLETED)):
             reasons = event['job-state-reasons']
             debugprint (reasons)
             if type (reasons) != list:
@@ -1962,8 +1982,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.update_status ()
 
     def state_reason_added (self, mon, reason):
-        monitor.Watcher.state_reason_added (self, mon, reason)
-
         (title, text) = reason.get_description ()
         printer = reason.get_printer ()
 
@@ -1978,7 +1996,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.update_status ()
         self.treeview.queue_draw ()
 
-        if not self.trayicon:
+        if not self.applet:
             return
 
         # Find out if the user has jobs queued for that printer.
@@ -1991,8 +2009,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
                 break
 
     def state_reason_removed (self, mon, reason):
-        monitor.Watcher.state_reason_removed (self, mon, reason)
-
         printer = reason.get_printer ()
         try:
             reasons = self.printer_state_reasons[printer]
@@ -2011,7 +2027,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.update_status ()
         self.treeview.queue_draw ()
 
-        if not self.trayicon:
+        if not self.applet:
             return
 
         tuple = reason.get_tuple ()
@@ -2031,16 +2047,13 @@ class JobViewer (GtkGUI, monitor.Watcher):
             pass
 
     def still_connecting (self, mon, reason):
-        monitor.Watcher.still_connecting (self, mon, reason)
-        if not self.trayicon:
+        if not self.applet:
             return
 
         self.notify_printer_state_reason (reason)
 
     def now_connected (self, mon, printer):
-        monitor.Watcher.now_connected (self, mon, printer)
-
-        if not self.trayicon:
+        if not self.applet:
             return
 
         # Find the connecting-to-device state reason.
@@ -2058,6 +2071,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
             tuple = reason.get_tuple ()
         else:
             debugprint ("Couldn't find state reason in list!")
+            tuple = None
             for (level,
                  p,
                  r) in self.state_reason_notifications.keys ():
@@ -2066,6 +2080,11 @@ class JobViewer (GtkGUI, monitor.Watcher):
                     tuple = (level, p, r)
                     break
 
+            if tuple == None:
+                debugprint ("Unexpected now_connected signal "
+                            "(reason not in notifications list)")
+                return
+
         try:
             notification = self.state_reason_notifications[tuple]
         except KeyError:
@@ -2081,15 +2100,12 @@ class JobViewer (GtkGUI, monitor.Watcher):
             notification.set_data ('closed', True)
 
     def printer_added (self, mon, printer):
-        monitor.Watcher.printer_added (self, mon, printer)
         self.printer_uri_index.add_printer (printer)
 
     def printer_event (self, mon, printer, eventname, event):
-        monitor.Watcher.printer_event (self, mon, printer, eventname, event)
         self.printer_uri_index.update_from_attrs (printer, event)
 
     def printer_removed (self, mon, printer):
-        monitor.Watcher.printer_removed (self, mon, printer)
         self.printer_uri_index.remove_printer (printer)
 
     ### Cell data functions
@@ -2252,24 +2268,4 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
         cell.set_property ("text", text)
 
-    def _on_treeview_query_tooltip (self, tv, x, y, keyboard_mode, tooltip):
-        if keyboard_mode:
-            (path, column) = tv.get_cursor ()
-            if path is None:
-                return False
-        else:
-            bin_x, bin_y = tv.convert_widget_to_bin_window_coords (x, y)
-            ret = tv.get_path_at_pos (bin_x, bin_y)
-            if ret is None:
-                return False
-            path = ret[0]
-            column = ret[1]
-
-        cells = column.get_cell_renderers ()
-        for cell in cells:
-            if type (cell) == gtk.CellRendererText:
-                tooltip.set_markup (cell.get_property ("text"))
-                self.treeview.set_tooltip_cell (tooltip, path, column, cell)
-                break
-
-        return True
+gobject.type_register (JobViewer)
