@@ -654,8 +654,12 @@ class Monitor(gobject.GObject):
         jobids.sort ()
         if got > 0:
             last_jobid = jobids[got - 1]
+            if last_jobid < self.fetch_first_job_id:
+                last_jobid = self.fetch_first_job_id + limit - 1
+                debugprint ("Unexpected job IDs returned: %s" % repr (jobids))
+                debugprint ("That's not what we asked for!")
         else:
-            last_jobid = self.fetch_first_job_id + limit
+            last_jobid = self.fetch_first_job_id + limit - 1
         for jobid in xrange (self.fetch_first_job_id, last_jobid + 1):
             try:
                 job = fetched[jobid]
