@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "gnu" -*-
  * udev-configure-printer - a udev callout to configure print queues
- * Copyright (C) 2009, 2010 Red Hat, Inc.
+ * Copyright (C) 2009, 2010, 2011 Red Hat, Inc.
  * Author: Tim Waugh <twaugh@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,9 +41,12 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
+#include <ctype.h>
 #include <syslog.h>
 #include <unistd.h>
 #include <usb.h>
+#include <glib.h>
 
 #define DISABLED_REASON "Unplugged or turned off"
 #define MATCH_ONLY_DISABLED 1
@@ -442,7 +445,8 @@ static char *
 device_id_from_devpath (const char *devpath,
 			const struct usb_uri_map *map,
 			struct device_id *id,
-			char *usbserial, size_t usbseriallen)
+			char *usbserial, size_t usbseriallen,
+			char *usblpdev, size_t usblpdevlen)
 {
   struct usb_uri_map_entry *entry;
   struct udev *udev;
@@ -460,6 +464,7 @@ device_id_from_devpath (const char *devpath,
   int conf = 0, iface = 0;
   int got = 0;
   char *usb_device_devpath;
+  char *usblpdevpos, *dest;
 
   id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
 
@@ -470,6 +475,31 @@ device_id_from_devpath (const char *devpath,
       exit (1);
     }
 
+  /* For devices discovered via the usblp kernel module we read out the number
+   * of the /dev/usb/lp* device file, as there can be queues set up with 
+   * non-standard CUPS backends based on the /dev/usb/lp* device file and
+   * we want to avoid that an additional queue with a standard CUPS backend
+   * gets set up.
+   */
+  usblpdevpos = strstr(devpath, "/usb/lp");
+  if (usblpdevpos != NULL)
+    usblpdevpos += 7;
+  else
+    {
+      usblpdevpos = strstr(devpath, "/usblp");
+      if (usblpdevpos != NULL)
+	usblpdevpos += 6;
+    }
+  if (usblpdevpos != NULL)
+    {
+      for (dest = usblpdev;
+	   (*usblpdevpos >= '0') && (*usblpdevpos <= '9') &&
+	     (dest - usblpdev < usblpdevlen);
+	   usblpdevpos ++, dest ++)
+	*dest = *usblpdevpos;
+      *dest = '\0';
+    }
+
   sys = udev_get_sys_path (udev);
   syslen = strlen (sys);
   devpathlen = strlen (devpath);
@@ -679,6 +709,48 @@ device_id_from_devpath (const char *devpath,
   return usb_device_devpath;
 }
 
+static void
+device_id_from_bluetooth (const char *bdaddr, struct device_id *id)
+{
+  gint exit_status;
+  char *device_id;
+  gchar *argv[4];
+
+  id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
+  argv[0] = g_strdup ("/usr/lib/cups/backend/bluetooth");
+  argv[1] = g_strdup ("--get-deviceid");
+  argv[2] = g_strdup (bdaddr);
+  argv[3] = NULL;
+  if (g_spawn_sync (NULL, argv, NULL,
+		    G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL,
+		    &device_id, NULL, &exit_status, NULL) == FALSE) {
+    g_free (argv[0]);
+    g_free (argv[1]);
+    g_free (argv[2]);
+    return;
+  }
+  g_free (argv[0]);
+  g_free (argv[1]);
+  g_free (argv[2]);
+
+  if (WEXITSTATUS(exit_status) == 0)
+    parse_device_id (device_id, id);
+
+  g_free (device_id);
+}
+
+static char *
+uri_from_bdaddr (const char *devpath)
+{
+  return g_strdup_printf("bluetooth://%c%c%c%c%c%c%c%c%c%c%c%c",
+			 devpath[0], devpath[1],
+			 devpath[3], devpath[4],
+			 devpath[6], devpath[7],
+			 devpath[9], devpath[10],
+			 devpath[12], devpath[13],
+			 devpath[15], devpath[16]);
+}
+
 static const char *
 no_password (const char *prompt)
 {
@@ -724,7 +796,9 @@ find_matching_device_uris (struct device_id *id,
   size_t i, n;
   const char *exclude_schemes[] = {
     "beh",
+    "cups-pdf",
     "bluetooth",
+    "dnssd",
     "http",
     "https",
     "ipp",
@@ -743,7 +817,7 @@ find_matching_device_uris (struct device_id *id,
   /* Leave the bus to settle. */
   sleep (1);
 
-  cups = httpConnectEncrypt ("localhost", 631, HTTP_ENCRYPT_IF_REQUESTED);
+  cups = httpConnectEncrypt (cupsServer (), ippPort(), cupsEncryption ());
   if (cups == NULL)
     {
       /* Don't bother retrying here.  Instead, the CUPS initscript
@@ -765,6 +839,8 @@ find_matching_device_uris (struct device_id *id,
   ippAddStrings (request, IPP_TAG_OPERATION, IPP_TAG_NAME, "exclude-schemes",
 		 sizeof (exclude_schemes) / sizeof(exclude_schemes[0]),
 		 NULL, exclude_schemes);
+  ippAddInteger (request, IPP_TAG_OPERATION, IPP_TAG_INTEGER, "timeout",
+                 2);
 
   answer = cupsDoRequestOrDie (cups, request, "/");
   httpClose (cups);
@@ -1003,17 +1079,76 @@ find_matching_device_uris (struct device_id *id,
   return uris->n_uris;
 }
 
+char *
+normalize_device_uri(const char *str_orig)
+{
+  int i, j;
+  int havespace = 0;
+  char *str;
+
+  if (str_orig == NULL)
+    return NULL;
+
+  str = strdup(str_orig);
+  for (i = 0, j = 0; i < strlen(str); i++, j++)
+    {
+      if (((str[i] >= 'A') && (str[i] <= 'Z')) ||
+	  ((str[i] >= 'a') && (str[i] <= 'z')) ||
+	  ((str[i] >= '0') && (str[i] <= '9')))
+	{
+	  /* Letter or number, keep it */
+	  havespace = 0;
+	  str[j] = tolower(str[i]);
+	}
+      else
+	{
+	  if ((str[i] == '%') && (i <= strlen(str)-3) &&
+	      (((str[i+1] >= 'A') && (str[i+1] <= 'F')) ||
+	       ((str[i+1] >= 'a') && (str[i+1] <= 'f')) ||
+	       ((str[i+1] >= '0') && (str[i+1] <= '9'))) &&
+	    (((str[i+2] >= 'A') && (str[i+2] <= 'F')) ||
+	     ((str[i+2] >= 'a') && (str[i+2] <= 'f')) ||
+	     ((str[i+2] >= '0') && (str[i+2] <= '9'))))
+	    /* Hex-encoded special characters replace by a single space if the
+	       last character is not already a space */
+	    i += 2;
+	  if (havespace == 1)
+	    j --;
+	  else
+	    {
+	      havespace = 1;
+	      str[j] = ' ';
+	    }
+	}
+    }
+  /* Add terminating zero */
+  str[j] = '\0';
+  /* Cut off trailing white space */
+  while (str[strlen(str)-1] == ' ')
+    str[strlen(str)-1] = '\0';
+  /* Cut off all before model name */
+  while ((strstr(str, "hp ") == str) ||
+	 (strstr(str, "hewlett ") == str) ||
+	 (strstr(str, "packard ") == str) ||
+	 (strstr(str, "apollo ") == str) ||
+	 (strstr(str, "usb ") == str))
+    str = strchr(str, ' ') + 1;
+
+  return str;
+}
+
 /* Call a function for each queue with the given device-uri and printer-state.
  * Returns the number of queues with a matching device-uri. */
 static size_t
 for_each_matching_queue (struct device_uris *device_uris,
 			 int flags,
 			 void (*fn) (const char *, void *),
-			 void *context)
+			 void *context,
+			 char *usblpdev, size_t usblpdevlen)
 {
   size_t matched = 0;
-  http_t *cups = httpConnectEncrypt ("localhost", 631,
-				     HTTP_ENCRYPT_IF_REQUESTED);
+  http_t *cups = httpConnectEncrypt (cupsServer (), ippPort (),
+				     cupsEncryption ());
   ipp_t *request, *answer;
   ipp_attribute_t *attr;
   const char *attributes[] = {
@@ -1022,6 +1157,7 @@ for_each_matching_queue (struct device_uris *device_uris,
     "printer-state",
     "printer-state-message",
   };
+  char usblpdevstr1[32] = "", usblpdevstr2[32] = "";
 
   if (cups == NULL)
     return 0;
@@ -1053,13 +1189,31 @@ for_each_matching_queue (struct device_uris *device_uris,
       exit (1);
     }
 
+  if (strlen(usblpdev) > 0)
+    {
+      /* If one of these strings is contained in one of the existing queue's
+	 device URIs, consider the printer as already configured. Some
+	 non-standard CUPS backend use the (obsolete) reference to the
+	 usblp device file. This avoids that in such a case a second queue
+	 with a standard CUPS backend is auto-created. */
+      snprintf(usblpdevstr1, sizeof(usblpdevstr1), "/usb/lp%s",
+	       usblpdev);
+      snprintf(usblpdevstr2, sizeof(usblpdevstr2), "/usblp%s",
+	       usblpdev);
+      syslog (LOG_DEBUG,
+	      "Consider also queues with \"%s\" or \"%s\" in their URIs as matching",
+	      usblpdevstr1, usblpdevstr2);
+    }
+
   for (attr = answer->attrs; attr; attr = attr->next)
     {
       const char *this_printer_uri = NULL;
       const char *this_device_uri = NULL;
       const char *printer_state_message = NULL;
       int state = 0;
-      size_t i;
+      size_t i, l;
+      char *this_device_uri_n, *device_uri_n;
+      const char *ps1, *ps2, *pi1, *pi2;
 
       while (attr && attr->group_tag != IPP_TAG_PRINTER)
 	attr = attr->next;
@@ -1084,21 +1238,70 @@ for_each_matching_queue (struct device_uris *device_uris,
 	    state = attr->values[0].integer;
 	}
 
-      for (i = 0; i < device_uris->n_uris; i++)
-	if (!strcmp (device_uris->uri[i], this_device_uri))
-	  {
-	    matched++;
-	    if (((flags & MATCH_ONLY_DISABLED) &&
-		 state == IPP_PRINTER_STOPPED &&
-		 !strcmp (printer_state_message, DISABLED_REASON)) ||
-		(flags & MATCH_ONLY_DISABLED) == 0)
-	      {
-		syslog (LOG_DEBUG ,"Queue %s has matching device URI",
-			this_printer_uri);
-		(*fn) (this_printer_uri, context);
-	      }
-	  }
+      if (!this_device_uri)
+	/* CUPS didn't include a device-uri attribute in the response
+	   for this printer (shouldn't happen). */
+	goto skip;
 
+      this_device_uri_n = normalize_device_uri(this_device_uri);
+      syslog (LOG_DEBUG, "URI of print queue: %s, normalized: %s",
+	      this_device_uri, this_device_uri_n);
+      pi1 = strstr (this_device_uri, "interface=");
+      ps1 = strstr (this_device_uri, "serial=");
+      for (i = 0; i < device_uris->n_uris; i++)
+	{
+	  device_uri_n = normalize_device_uri(device_uris->uri[i]);
+	  /* As for the same device different URIs can come out when the
+	     device is accessed via the usblp kernel module or via low-
+	     level USB (libusb) we cannot simply compare URIs, must
+	     consider also URIs as equal if one has an "interface"
+	     or "serial" attribute and the other not. If both have
+	     the attribute it must naturally match. We check which attributes 
+             are there and this way determine up to which length the two URIs 
+             must match. Here we can assume that if a URI has an "interface"
+	     attribute it has also a "serial" attribute, as this URI is
+	     an URI obtained via libusb and these always have a "serial"
+	     attribute. usblp-based URIs never have an "interface"
+	     attribute.*/
+	  pi2 = strstr (device_uris->uri[i], "interface=");
+	  ps2 = strstr (device_uris->uri[i], "serial=");
+	  if (pi1 && !pi2)
+	    l = strlen(device_uris->uri[i]);
+	  else if (!pi1 && pi2)
+	    l = strlen(this_device_uri);
+	  else if (ps1 && !ps2)
+	    l = strlen(device_uris->uri[i]);
+	  else if (!ps1 && ps2)
+	    l = strlen(this_device_uri);
+	  else if (strlen(this_device_uri) > strlen(device_uris->uri[i]))
+	    l = strlen(this_device_uri);
+	  else
+	    l = strlen(device_uris->uri[i]);
+	  syslog (LOG_DEBUG, "URI of detected printer: %s, normalized: %s",
+		  device_uris->uri[i], device_uri_n);
+	  if ((!strncmp (device_uris->uri[i], this_device_uri, l)) ||
+	      (strstr (device_uri_n, this_device_uri_n) == 
+	       device_uri_n) ||
+	      (strstr (this_device_uri_n, device_uri_n) == 
+	       this_device_uri_n) ||
+	      ((strlen(usblpdev) > 0) &&
+	       ((strstr (this_device_uri, usblpdevstr1) != NULL) ||
+	       (strstr (this_device_uri, usblpdevstr2) != NULL)))) 
+	    {
+	      matched++;
+	      syslog (LOG_DEBUG, "Queue %s has matching device URI",
+		      this_printer_uri);
+	      if (((flags & MATCH_ONLY_DISABLED) &&
+		   state == IPP_PRINTER_STOPPED &&
+		   !strcmp (printer_state_message, DISABLED_REASON)) ||
+		  (flags & MATCH_ONLY_DISABLED) == 0)
+		{
+		  (*fn) (this_printer_uri, context);
+		}
+	    }
+	}
+
+    skip:
       if (!attr)
 	break;
     }
@@ -1110,9 +1313,9 @@ for_each_matching_queue (struct device_uris *device_uris,
 static void
 enable_queue (const char *printer_uri, void *context)
 {
-  /* Disable it. */
-  http_t *cups = httpConnectEncrypt ("localhost", 631,
-				     HTTP_ENCRYPT_IF_REQUESTED);
+  /* Enable it. */
+  http_t *cups = httpConnectEncrypt (cupsServer (), ippPort (),
+				     cupsEncryption ());
   ipp_t *request, *answer;
 
   if (cups == NULL)
@@ -1138,6 +1341,38 @@ enable_queue (const char *printer_uri, void *context)
   httpClose (cups);
 }
 
+static gboolean
+bluetooth_verify_address (const char *bdaddr)
+{
+  gboolean retval = TRUE;
+  char **elems;
+  guint i;
+
+  g_return_val_if_fail (bdaddr != NULL, FALSE);
+
+  if (strlen (bdaddr) != 17)
+    return FALSE;
+
+  elems = g_strsplit (bdaddr, ":", -1);
+  if (elems == NULL)
+    return FALSE;
+  if (g_strv_length (elems) != 6) {
+    g_strfreev (elems);
+    return FALSE;
+  }
+  for (i = 0; i < 6; i++) {
+    if (strlen (elems[i]) != 2 ||
+        g_ascii_isxdigit (elems[i][0]) == FALSE ||
+        g_ascii_isxdigit (elems[i][1]) == FALSE) {
+	  retval = FALSE;
+	  break;
+    }
+  }
+
+  g_strfreev (elems);
+  return retval;
+}
+
 static int
 do_add (const char *cmd, const char *devpath)
 {
@@ -1146,8 +1381,10 @@ do_add (const char *cmd, const char *devpath)
   struct device_id id;
   struct device_uris device_uris;
   struct usb_uri_map *map;
-  char *usb_device_devpath;
+  char *usb_device_devpath = NULL;
   char usbserial[256];
+  char usblpdev[8] = "";
+  gboolean is_bluetooth;
 
   if (getenv ("DEBUG") == NULL)
     {
@@ -1173,33 +1410,53 @@ do_add (const char *cmd, const char *devpath)
 
   syslog (LOG_DEBUG, "add %s", devpath);
 
+  is_bluetooth = bluetooth_verify_address (devpath);
+
   map = read_usb_uri_map ();
-  usb_device_devpath = device_id_from_devpath (devpath, map, &id,
-					       usbserial, sizeof (usbserial));
+  if (is_bluetooth) {
+    usbserial[0] = '\0';
+    device_id_from_bluetooth (devpath, &id);
+  } else {
+    usb_device_devpath = device_id_from_devpath (devpath, map, &id,
+						 usbserial, sizeof (usbserial),
+						 usblpdev, sizeof (usblpdev));
+  }
 
   if (!id.mfg || !id.mdl)
     {
-      syslog (LOG_ERR, "invalid or missing IEEE 1284 Device ID%s%s",
+      syslog (LOG_DEBUG, "invalid or missing IEEE 1284 Device ID%s%s",
 	      id.full_device_id ? " " : "",
 	      id.full_device_id ? id.full_device_id : "");
       exit (1);
     }
 
   syslog (LOG_DEBUG, "MFG:%s MDL:%s SERN:%s serial:%s", id.mfg, id.mdl,
-	  id.sern ? id.sern : "-", usbserial);
+	  id.sern ? id.sern : "-", usbserial[0] ? usbserial : "-");
+
+  if (!is_bluetooth)
+    {
+      find_matching_device_uris (&id, usbserial, &device_uris, usb_device_devpath,
+				 map);
+      free (usb_device_devpath);
+    } else {
+      char *device_uri;
+
+      device_uri = uri_from_bdaddr (devpath);
+      add_device_uri (&device_uris, device_uri);
+      g_free (device_uri);
+    }
 
-  find_matching_device_uris (&id, usbserial, &device_uris, usb_device_devpath,
-			     map);
-  free (usb_device_devpath);
   if (device_uris.n_uris == 0)
     {
+      syslog (LOG_ERR, "no corresponding CUPS device found");
       free_device_id (&id);
       return 0;
     }
 
   /* Re-enable any queues we'd previously disabled. */
   if (for_each_matching_queue (&device_uris, MATCH_ONLY_DISABLED,
-			       enable_queue, NULL) == 0)
+			       enable_queue, NULL,
+			       usblpdev, sizeof (usblpdev)) == 0)
     {
       size_t i;
       int type;
@@ -1245,12 +1502,44 @@ do_add (const char *cmd, const char *devpath)
   return 0;
 }
 
+static void
+remove_queue (const char *printer_uri)
+{
+  /* Disable it. */
+  http_t *cups = httpConnectEncrypt (cupsServer (), ippPort (),
+				     cupsEncryption ());
+  ipp_t *request, *answer;
+
+  if (cups == NULL)
+    return;
+
+  request = ippNewRequest (CUPS_DELETE_PRINTER);
+  ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI,
+		"printer-uri", NULL, printer_uri);
+  answer = cupsDoRequest (cups, request, "/admin/");
+  if (!answer)
+    {
+      syslog (LOG_ERR, "Failed to send IPP-Delete-Printer request");
+      httpClose (cups);
+      return;
+    }
+
+  if (answer->request.status.status_code > IPP_OK_CONFLICT)
+    syslog (LOG_ERR, "IPP-Delete-Printer request failed");
+  else
+    syslog (LOG_INFO, "Deleted printer %s as the corresponding device "
+	    "was unpaired", printer_uri);
+
+  ippDelete (answer);
+  httpClose (cups);
+}
+
 static void
 disable_queue (const char *printer_uri, void *context)
 {
   /* Disable it. */
-  http_t *cups = httpConnectEncrypt ("localhost", 631,
-				     HTTP_ENCRYPT_IF_REQUESTED);
+  http_t *cups = httpConnectEncrypt (cupsServer (), ippPort (),
+				     cupsEncryption ());
   ipp_t *request, *answer;
 
   if (cups == NULL)
@@ -1285,8 +1574,19 @@ do_remove (const char *devpath)
   struct usb_uri_map *map;
   struct usb_uri_map_entry *entry, **prev;
   struct device_uris *uris = NULL;
+  char usblpdev[8] = "";
   syslog (LOG_DEBUG, "remove %s", devpath);
 
+  if (bluetooth_verify_address (devpath))
+    {
+      char *device_uri;
+
+      device_uri = uri_from_bdaddr (devpath);
+      remove_queue (devpath);
+      g_free (device_uri);
+      return 0;
+    }
+
   map = read_usb_uri_map ();
   prev = &map->entries;
   for (entry = map->entries; entry; entry = entry->next)
@@ -1303,7 +1603,8 @@ do_remove (const char *devpath)
   if (uris)
     {
       /* Find the relevant queues and disable them if they are enabled. */
-      for_each_matching_queue (uris, 0, disable_queue, NULL);
+      for_each_matching_queue (uris, 0, disable_queue, NULL,
+			       usblpdev, sizeof (usblpdev));
       *prev = entry->next;
       write_usb_uri_map (map);
     }
